#include "kanji_union.h"

extern EUC_TABLE *u;
KANJI kanji_union;
KANJI *double_byte_character = &kanji_union;
NORMAL_SUBSTITUTE data[DATA_ARRAY_LIMIT];

static int ERROR_CODE = -1;

int inverted(char *c) {
    char *p = (char *)double_byte_character->sequence;

// Copying in reverse order
    p[0] = c[1];
    p[1] = c[0];
    return double_byte_character->code[0];
}

int kanjiCode(char *q) {
    BOOL kanji = FALSE;
    char *buffer;
    char *p;
    int before;
    int i, k, n;
    int size = strlen(q);
    int size_1 = size + 1;
    unsigned char uc;

    buffer = malloc(size_1);
    if(buffer == NULL) {
        fprintf(stderr, "malloc() error-3: %d bytes\n", size_1);
        return ERROR_CODE;
    }

    strcpy(buffer, q);

    for(i = 0; buffer[i]; ++i) {
        uc = (unsigned char)buffer[i];
        k = (int)uc;

        if(kanji) {
            p = & buffer[before];
            n = inverted(p);
            break;
        }

        if(0x81 <= k && k <= 0xFC) {
            if (KANA_KUTEN <= k && k <= KANA_HANDAKUTEN) {
                n = k;
                break;
            } else {
                before = i;
                kanji = TRUE;
            }

        } else {
            n = k;
            break;
        }

    }

    free(buffer);
    return n;
}

BOOL edit_record(char *normal, char *substitute, char *binary_name, BOOL no_searching, BOOL *which_table) {
    char *combo;
    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    FILE *file;
    FILE *edited_file;
    int buffer_size = BLOCK_SIZE;
    int from, i, into, k;
    int n = 0;
    unsigned char uc;
    OPEN_CLOSE_FUNCTION *open_close = malloc( sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
    if(open_close == NULL) {
        fprintf(stderr, "malloc() error-4: %d bytes", sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
        return FALSE;
    }

    open_close[0] = binary_file_in_installed_directory;
    open_close[1] = binary_file_in_current_directory;
    open_close[2] = binary_file_in_path_directory;

    for(i = 0; i < SEARCH_COUNT; ++i) {
        if( open_close[i](buffer, buffer_size) ) {
            if(*which_table) {
                fprintf(stdout, "%s is primarily accessible to diyo.\n", buffer);
                *which_table = FALSE;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    fprintf(stderr, "open_close[ %d ]() error: %s\n", i, SJIS_EUC_BIN);
                    free(open_close);
                    return FALSE;
                }

                break;
            case 2:
                fprintf(stderr, "open_close[ %d ]() error: %s\n", i, SJIS_EUC_BIN);
                free(open_close);
                return FALSE;
            default: // case 1:
                break;
            }

        }

    }

    if(! setting(u, no_searching, which_table) ) {
        error_handler("edit_record() error:", "setting(u, r->no_searching, which_table)");
        free(open_close);
        return FALSE;
    }

    from = kanjiCode(normal);
    into = kanjiCode(substitute);
    fprintf(stdout, "%04X %s %04X \n", from, RIGHT_ARROW, into);

    combo = malloc(COMBO_SIZE);
    if(combo == NULL) {
        fprintf(stderr, "malloc() error-5: %d bytes\n", COMBO_SIZE);
        return FALSE;
    }

    file = fopen(buffer, "rb");
    for(i = 1; i <= RECORD_COUNT; ++i) {
        fread(c, DOUBLE_KANJI, 1, file);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            k = (int)uc;
        } else {
            k = inverted(c);
        }

        combo[n++] = c[0];
        combo[n++] = c[1];

        if(k == from) {
            combo[n++] = working(u, into, 0);
            combo[n++] = working(u, into, 1);
        } else {
            combo[n++] = c[SJIS_KANJI_SIZE];
            combo[n++] = c[ SJIS_KANJI_SIZE + 1 ];
        }

    }

    fclose(file);

    edited_file = fopen(binary_name, "wb");
    fwrite(combo, n, 1, edited_file);
    fclose(edited_file);

    free(combo);
    free(open_close);
    return TRUE;
}

BOOL edit_many_records(char *text, char *binary_name, BOOL no_searching, BOOL *which_table) {
    BOOL checked = FALSE;
    char *b;
    char *combo;
    char *p;
    char *q;
    char *r;
    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    char record[BLOCK_SIZE];
    FILE *edited_file;
    FILE *file;
    FILE *text_file;
    int buffer_size = BLOCK_SIZE;
    int from, i, into, k, m;
    int n = 0;
    int size = 0;
    int used = 0;
    unsigned char uc;
    OPEN_CLOSE_FUNCTION *open_close = malloc( sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
    if(open_close == NULL) {
        fprintf(stderr, "malloc() error-6: %d bytes", sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
        return FALSE;
    }

    open_close[0] = binary_file_in_installed_directory;
    open_close[1] = binary_file_in_current_directory;
    open_close[2] = binary_file_in_path_directory;

    for(i = 0; i < SEARCH_COUNT; ++i) {
        if( open_close[i](buffer, buffer_size) ) {
            if(*which_table) {
                fprintf(stdout, "%s is primarily accessible to diyo.\n", buffer);
                *which_table = FALSE;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    fprintf(stderr, "open_close[ %d ]() error: %s\n", i, SJIS_EUC_BIN);
                    free(open_close);
                    return FALSE;
                }

                break;
            case 2:
                fprintf(stderr, "open_close[ %d ]() error: %s\n", i, SJIS_EUC_BIN);
                free(open_close);
                return FALSE;
            default: // case 1:
                break;
            }

        }

    }

    if(! setting(u, no_searching, which_table) ) {
        error_handler("edit_many_records() error:", "setting(u, r->no_searching, which_table)");
        free(open_close);
        return FALSE;
    }

    text_file = fopen(text, "rt");
    if(text_file == NULL) {
        fprintf(stderr, "fopen() error: %s\n", text);
        free(open_close);
        return FALSE;
    }

    for(m = 0; m < DATA_ARRAY_LIMIT; m++) {
        b = fgets(record, BLOCK_SIZE, text_file);
        if(b == NULL) {
            break;
        } else {
            b = strchr(record, '\n');
            if(b != NULL) {
                *b = '\0';
            }

        }

        size = strlen(record);
        if(size == DUAL_KANJI) {
            data[m].please_check = TRUE;
            p = & data[m].normal[0];
            r = & record[0];
            memcpy(p, r, 2);
            p[2] = '\0';

            q = & data[m].substitute[0];
            r = & record[SJIS_KANJI_SIZE];
            memcpy(q, r, 2);
            q[2] = '\0';
        } else {
            data[m].please_check = TRUE;
            p = & data[m].normal[0];
            p[0] = record[0]; // katakana, for example
            p[1] = '\0';
            p[2] = '\0';

            q = & data[m].substitute[0];
            q[0] = record[KATAKANA_SIZE];
            q[1] = '\0';
            q[2] = '\0';
        }

        used = m;
    }

    fclose(text_file);

    combo = malloc(COMBO_SIZE);
    if(combo == NULL) {
        fprintf(stderr, "malloc() error-7: %d bytes\n", COMBO_SIZE);
        return FALSE;
    }

    file = fopen(buffer, "rb");
    for(i = 1; i <= RECORD_COUNT; ++i) {
        fread(c, DOUBLE_KANJI, 1, file);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            k = (int)uc;
        } else {
            k = inverted(c);
        }

        combo[n++] = c[0];
        combo[n++] = c[1];

        for(m = 0; m <= used; ++m) {
            if(data[m].please_check) {
                from = kanjiCode(data[m].normal);
                if(k == from) {
                    into = kanjiCode(data[m].substitute);
                    checked = TRUE;
                    data[m].please_check = FALSE;
                    break;
                } else {
                    checked = FALSE;
                }

            } else {
                checked = FALSE;
            }

        }

        if(checked) {
            combo[n++] = working(u, into, 0);
            combo[n++] = working(u, into, 1);
        } else {
            combo[n++] = c[SJIS_KANJI_SIZE];
            combo[n++] = c[ SJIS_KANJI_SIZE + 1 ];
        }

    }

    fclose(file);

    edited_file = fopen(binary_name, "wb");
    fwrite(combo, n, 1, edited_file);
    fclose(edited_file);

    free(combo);
    free(open_close);
    return TRUE;
}

void error_handler(const char *p, const char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

